import sys


class Progress:
    """A class for creating progress updates for long running operations."""

    should_output = True

    def __init__(self, total=None, message='', increment=1, multiplier=1, start=0, callback=None, force=False):
        self.total = total
        self.message = message
        self.increment = increment
        self.multiplier = multiplier
        self.current = start
        self.callback = callback if callback else str
        self.should_output = force or Progress.should_output
        self._next = increment

    def iterator(self, iterable):
        """Iterates over iterable and automatically updates the status at the predefined increments."""
        if self.should_output:
            self.show()
            i = 0
            for n in iterable:
                i += self.multiplier
                yield n
                if i == self.increment:
                    self.current += i
                    i = 0
                    self.show()
            self.current += i
            self.finish()
        else:
            yield from iterable

    def inc(self, amount=1):
        self.current += amount
        if self.current >= self._next:
            self.show()
            self._next += self.increment

    def finish(self):
        self.show()
        sys.stderr.write('\n')

    def show(self):
        if self.total:
            sys.stderr.write('\r\033[2K{:s} {:.2%} ({:,d} / {:,d}). {:s}'.format(self.message, self.current / self.total, self.current, self.total, self.callback()))
        else:
            sys.stderr.write('\r\033[2K{:s} {:,d}. {:s}'.format(self.message, self.current, self.callback()))

    @staticmethod
    def set_output(b):
        Progress.should_output = b

    @staticmethod
    def message(*args, **kwargs):
        if Progress.should_output:
            print(*args, **kwargs)



import json
from collections import defaultdict

from traceutils.radix.ip2as import IP2AS


class IX:
    def __init__(self, city, country, created, id, media, name, name_long, notes, org_id, policy_email, policy_phone, proto_ipv6, proto_multicast, proto_unicast, region_continent, status, tech_email, tech_phone, updated, url_stats, website):
        self.city = city
        self.country = country
        self.created = created
        self.id = id
        self.media = media
        self.name = name
        self.name_long = name_long
        self.notes = notes
        self.org_id = org_id
        self.policy_email = policy_email
        self.policy_phone = policy_phone
        self.proto_ipv6 = proto_ipv6
        self.proto_multicast = proto_multicast
        self.proto_unicast = proto_unicast
        self.region_continent = region_continent
        self.status = status
        self.tech_email = tech_email
        self.tech_phone = tech_phone
        self.updated = updated
        self.url_stats = url_stats
        self.website = website

    def __repr__(self):
        return '<IX {}>'.format(self.name)


class IXLAN:
    def __init__(self, ix, arp_sponge, created, descr, dot1q_support, id, ix_id, mtu, name, rs_asn, status, updated):
        self.ix = ix
        self.arp_sponge = arp_sponge
        self.created = created
        self.descr = descr
        self.dot1q_support = dot1q_support
        self.id = id
        self.ix_id = ix_id
        self.mtu = mtu
        self.name = name
        self.rs_asn = rs_asn
        self.status = status
        self.updated = updated


class IXPFX:
    def __init__(self, ixlan, created, id, ixlan_id, prefix, protocol, status, updated):
        self.ixlan = ixlan
        self.created = created
        self.id = id
        self.ixlan_id = ixlan_id
        self.prefix = prefix
        self.protocol = protocol
        self.status = status
        self.updated = updated

    def __repr__(self):
        return '<IXPFX Name={}, Prefix={}>'.format(self.ixlan.ix.name, self.prefix)


class NetIXLAN:
    def __init__(self, ixlan, asn, created, id, ipaddr4, ipaddr6, is_rs_peer, ix_id, ixlan_id, name, net_id, notes, speed,
                 status, updated):
        self.ix = ixlan.ix
        self.ixlan = ixlan
        self.asn = asn
        self.created = created
        self.id = id
        self.ipaddr4 = ipaddr4
        self.ipaddr6 = ipaddr6
        self.is_rs_peer = is_rs_peer
        self.ix_id = ix_id
        self.ixlan_id = ixlan_id
        self.name = name
        self.net_id = net_id
        self.notes = notes
        self.speed = speed
        self.status = status
        self.updated = updated

    def __repr__(self):
        return '<NetIXLAN ASN={}, IX={}>'.format(self.asn, self.ix.name)


class PeeringDB:

    def __init__(self, filename):
        with open(filename) as f:
            j = json.load(f)
        self.ixs = {ix['id']: IX(**ix) for ix in j['ix']['data']}
        self.ixlans = {ixlan['id']: IXLAN(self.ixs[ixlan['ix_id']], **ixlan) for ixlan in j['ixlan']['data']}
        self.ixpfxs = {ixpfx['id']: IXPFX(self.ixlans[ixpfx['ixlan_id']], **ixpfx) for ixpfx in j['ixpfx']['data']}
        self.netixlans = {netixlan['id']: NetIXLAN(self.ixlans[netixlan['ixlan_id']], **netixlan) for netixlan in j['netixlan']['data']}
        self.prefixes = {ixpfx.prefix: ixpfx.ixlan.ix.id for ixpfx in self.ixpfxs.values()}
        self.addrs = {}
        self.new_prefixes = {}
        trie = IP2AS()
        trie.add_private()
        for prefix in self.prefixes:
            trie.add(prefix, asn=1)
        for netixlan in self.netixlans.values():
            if netixlan.ipaddr4:
                asn = trie[netixlan.ipaddr4]
                if asn == 0:
                    prefix = netixlan.ipaddr4.rpartition('.')[0]
                    ixid = netixlan.ix.id
                    self.prefixes['{}.0/24'.format(prefix)] = ixid
                self.addrs[netixlan.ipaddr4] = netixlan.asn
            if netixlan.ipaddr6:
                asn = trie[netixlan.ipaddr6]
                if asn == 0:
                    prefix = netixlan.ipaddr6.rpartition(':')[0]
                    ixid = netixlan.ix.id
                    self.prefixes['{}:0/64'.format(prefix)] = ixid
                self.addrs[netixlan.ipaddr6] = netixlan.asn



from Cython.Distutils import build_ext
from setuptools import setup, find_packages
from setuptools.extension import Extension
from Cython.Build import cythonize


extensions_names = {
    'traceutils.utils.utils': ['traceutils/utils/utils.pyx'],
    'traceutils.utils.net': ['traceutils/utils/net.pyx'],
    'traceutils.utils.dicts': ['traceutils/utils/dicts.pyx'],
    'traceutils.file2.file2': ['traceutils/file2/file2.pyx'],
    'traceutils.as2org.as2org': ['traceutils/as2org/as2org.pyx'],
    'traceutils.bgp.bgp': ['traceutils/bgp/bgp.pyx'],
    'traceutils.bgpreader.reader': ['traceutils/bgpreader/reader.pyx'],
    'traceutils.radix.radix_prefix': ['traceutils/radix/radix_prefix.pyx'],
    'traceutils.radix.radix_node': ['traceutils/radix/radix_node.pyx'],
    'traceutils.radix.radix_tree': ['traceutils/radix/radix_tree.pyx'],
    'traceutils.radix.radix': ['traceutils/radix/radix.pyx'],
    'traceutils.radix.ip2as': ['traceutils/radix/ip2as.pyx'],
    'traceutils.scamper.hop': ['traceutils/scamper/hop.pyx'],
    'traceutils.scamper.atlas': ['traceutils/scamper/atlas.pyx'],
    'traceutils.scamper.warts': ['traceutils/scamper/warts.pyx'],
    'traceutils.progress.bar': ['traceutils/progress/bar.py']
}

extensions = [Extension(k, v) for k, v in extensions_names.items()]
package_data = {k: ['*.pxd'] for k in extensions_names}

setup(
    name="traceutils",
    version='0.0.2',
    packages=find_packages(),
    install_requires=['ujson', 'cython'],
    cmdclass={'build_ext': build_ext},
    ext_modules=cythonize(
        extensions,
        compiler_directives={
            'language_level': '3',
            'embedsignature': True
        },
        annotate=True
    ),
    zip_safe=False,
    package_data=package_data,
    include_package_data=True
)
